@@ -89,71 +89,6 @@ tracker_data_update_get_new_service_id (TrackerDBInterface *iface)
 	return ++max;
 }
 
-void
-tracker_data_update_increment_stats (TrackerDBInterface *iface,
-				     TrackerService     *service)
-{
-	const gchar *service_type, *parent;
-
-	service_type = tracker_service_get_name (service);
-	parent = tracker_service_get_parent (service);
-
-	tracker_db_interface_execute_procedure (iface,
-						NULL,
-						"IncStat",
-						service_type,
-						NULL);
-
-	if (parent) {
-		tracker_db_interface_execute_procedure (iface,
-							NULL,
-							"IncStat",
-							parent,
-							NULL);
-	}
-}
-
-void
-tracker_data_update_decrement_stats (TrackerDBInterface *iface,
-				     TrackerService     *service)
-{
-	const gchar *service_type, *parent;
-
-	service_type = tracker_service_get_name (service);
-	parent = tracker_service_get_parent (service);
-
-	tracker_db_interface_execute_procedure (iface,
-						NULL,
-						"DecStat",
-						service_type,
-						NULL);
-
-	if (parent) {
-		tracker_db_interface_execute_procedure (iface,
-							NULL,
-							"DecStat",
-							parent,
-							NULL);
-	}
-}
-
-void
-tracker_data_update_create_event (TrackerDBInterface *iface,
-				  guint32 service_id,
-				  const gchar *type)
-{
-	gchar *service_id_str;
-
-	service_id_str = tracker_guint32_to_string (service_id);
-
-	tracker_db_interface_execute_procedure (iface, NULL, "CreateEvent",
-						service_id_str,
-						type,
-						NULL);
-
-	g_free (service_id_str);
-}
-
 gboolean
 tracker_data_update_create_service (TrackerService *service,
 				    guint32	    service_id,
@@ -163,7 +98,7 @@ tracker_data_update_create_service (TrackerService *service,
 {
 	TrackerDBInterface *iface;
 	TrackerDBResultSet *result_set;
-	guint32	volume_id = 0;
+	guint32	volume_id = 1;
 	gchar *id_str, *service_type_id_str, *path, *volume_id_str;
 	gboolean is_dir, is_symlink;
 
@@ -217,6 +152,34 @@ tracker_data_update_create_service (TrackerService *service,
 	return TRUE;
 }
 
+void
+tracker_data_update_disable_service (TrackerService *service,
+				     guint32         service_id)
+{
+	TrackerDBInterface *iface;
+	gchar *service_id_str;
+
+	g_return_if_fail (TRACKER_IS_SERVICE (service));
+	g_return_if_fail (service_id >= 1);
+
+	iface = tracker_db_manager_get_db_interface_by_type (tracker_service_get_name (service),
+							     TRACKER_DB_CONTENT_TYPE_METADATA);
+
+	service_id_str = tracker_guint32_to_string (service_id);
+
+	tracker_db_interface_execute_procedure (iface,
+						NULL,
+						"DisableService",
+						service_id_str,
+						NULL);
+	tracker_db_interface_execute_procedure (iface,
+						NULL,
+						"MarkServiceForRemoval",
+						service_id_str,
+						NULL);
+	g_free (service_id_str);
+}
+
 void
 tracker_data_update_delete_service (TrackerService *service,
 				    guint32	    service_id)
@@ -240,6 +203,12 @@ tracker_data_update_delete_service (TrackerService *service,
 						service_id_str,
 						NULL);
 
+	/* Delete from cleanup maintenance table */
+	tracker_db_interface_execute_procedure (iface,
+						NULL,
+						"UnmarkServiceForRemoval",
+						service_id_str,
+						NULL);
 	g_free (service_id_str);
 }
 
@@ -358,9 +327,14 @@ tracker_data_update_set_metadata (TrackerService *service,
 {
 	TrackerDBInterface *iface;
 	gint metadata_key;
+	gint collate_key;
 	gchar *id_str;
 
+	if(!strlen(value))
+		return;
+
 	id_str = tracker_guint32_to_string (service_id);
+
 	iface = tracker_db_manager_get_db_interface_by_type (tracker_service_get_name (service),
 							     TRACKER_DB_CONTENT_TYPE_METADATA);
 
@@ -383,6 +357,7 @@ tracker_data_update_set_metadata (TrackerService *service,
 							tracker_field_get_id (field),
 							parsed_value,
 							value,
+							value,
 							NULL);
 		break;
 
@@ -430,6 +405,22 @@ tracker_data_update_set_metadata (TrackerService *service,
 						    service_id);
 	}
 
+	collate_key = tracker_ontology_service_get_key_collate (tracker_service_get_name (service),
+								tracker_field_get_name (field));
+	if (collate_key > 0) {
+		gchar *val;
+		
+		val = tracker_escape_string (value);
+		
+		tracker_db_interface_execute_query (iface, NULL,
+			       "update Services set KeyMetadataCollation%d = CollateKey('%s') where id = %d",
+			       collate_key,
+			       val,
+			       service_id);
+
+		g_free (val);
+	}
+
 	g_free (id_str);
 }
 
@@ -554,14 +545,6 @@ tracker_data_update_delete_content (TrackerService *service,
 	g_free (service_id_str);
 }
 
-void
-tracker_data_update_delete_handled_events (TrackerDBInterface *iface)
-{
-	g_return_if_fail (TRACKER_IS_DB_INTERFACE (iface));
-
-	tracker_data_manager_exec (iface, "DELETE FROM Events WHERE BeingHandled = 1");
-}
-
 void
 tracker_data_update_delete_service_by_path (const gchar *path,
 					    const gchar *rdf_type)
@@ -576,6 +559,11 @@ tracker_data_update_delete_service_by_path (const gchar *path,
 		return;
 
 	service = tracker_ontology_get_service_by_name (rdf_type);
+
+	if (!service) {
+		return;
+	}
+
 	service_type = tracker_service_get_name (service);
 	service_id = tracker_data_query_file_id (service_type, path);
 
@@ -707,28 +695,26 @@ tracker_data_update_replace_service (const gchar *path,
 	TrackerDBInterface  *iface;
 	TrackerDBResultSet  *result_set;
 	const gchar         *modified;
-	GError              *error = NULL;
 	TrackerService      *service;
 	gchar               *escaped_path;
 	gchar               *dirname;
 	gchar               *basename;
+	time_t               file_mtime;
+	gboolean             set_metadata = FALSE;
+	guint32              id = 0;
 
 	g_return_if_fail (path != NULL);
 	g_return_if_fail (metadata != NULL);
 
-	/* When merging from the decomposed branch to trunk then pick the version
-	 * in the decomposed branch for this function. However, carefully 
-	 * compare the features, as this version is more recent and has 
-	 * implemented a few significant items, whereas the version in the
-	 * decomposed branch was a proof of concept implementation, and might
-	 * not have these needed features. 
-	 */
-
 	if (!rdf_type)
 		return;
 
 	service = tracker_ontology_get_service_by_name (rdf_type);
 
+	if (!service) {
+		return;
+	}
+
 	iface = tracker_db_manager_get_db_interface_by_type (tracker_service_get_name (service),
 							     TRACKER_DB_CONTENT_TYPE_METADATA);
 
@@ -738,81 +724,58 @@ tracker_data_update_replace_service (const gchar *path,
 		return;
 	}
 
+	file_mtime = atoi (modified);
 	escaped_path = tracker_escape_string (path);
 
 	basename = g_path_get_basename (escaped_path);
 	dirname = g_path_get_dirname (escaped_path);
 
-	/* TODO Warning: comparing Modified against Accessed. Do we have a
-	 * better field for this? */
-
-	result_set = tracker_db_interface_execute_query (iface, &error,
-							 "SELECT ID, Accessed < '%s' FROM Services "
-							 "WHERE Path = '%s' AND "
-							 "Name = '%s'",
-							 modified,
-							 dirname, basename);
-
-	if (error) {
-		g_error_free (error);
-	}
-
+	result_set = tracker_db_interface_execute_procedure (iface, NULL,
+							     "GetServiceID",
+							     dirname,
+							     basename,
+							     NULL);
 	if (result_set) {
-		GValue id_value = { 0, };
-		GValue is_value = { 0, };
-		gint   iid_value, iis_value;
+		guint mtime;
 
-		_tracker_db_result_set_get_value (result_set, 0, &id_value);
-		iid_value = g_value_get_int (&id_value);
+		tracker_db_result_set_get (result_set,
+					   0, &id,
+					   1, &mtime,
+					   -1);
 
-		_tracker_db_result_set_get_value (result_set, 1, &is_value);
-		iis_value = g_value_get_int (&is_value);
-
-		if (iis_value) {
-			ForeachInMetadataInfo *info = g_slice_new (ForeachInMetadataInfo);
-			info->service = service;
-			info->iid_value = iid_value;
-
-			info->config = tracker_data_manager_get_config ();
-			info->language = tracker_data_manager_get_language ();
-
-			g_hash_table_foreach (metadata, 
-					      foreach_in_metadata_set_metadata,
-					      info);
-
-			g_slice_free (ForeachInMetadataInfo, info);
+		if (mtime != file_mtime) {
+			set_metadata = TRUE;
 		}
 
-		g_value_unset (&id_value);
-		g_value_unset (&is_value);
-
 		g_object_unref (result_set);
 	} else {
-		guint32     id;
-
 		id = tracker_data_update_get_new_service_id (iface);
 
 		if (tracker_data_update_create_service (service, id,
 							dirname, basename,
 							metadata)) {
-			ForeachInMetadataInfo *info;
-
-			info = g_slice_new (ForeachInMetadataInfo);
-
-			info->service = service;
-			info->iid_value = id;
-
-			info->config = tracker_data_manager_get_config ();
-			info->language = tracker_data_manager_get_language ();
-
-			g_hash_table_foreach (metadata, 
-					      foreach_in_metadata_set_metadata,
-					      info);
-
-			g_slice_free (ForeachInMetadataInfo, info);
+			set_metadata = TRUE;
 		}
 	}
 
+	if (set_metadata) {
+		ForeachInMetadataInfo *info;
+
+		info = g_slice_new (ForeachInMetadataInfo);
+
+		info->service = service;
+		info->iid_value = id;
+
+		info->config = tracker_data_manager_get_config ();
+		info->language = tracker_data_manager_get_language ();
+
+		g_hash_table_foreach (metadata,
+				      foreach_in_metadata_set_metadata,
+				      info);
+
+		g_slice_free (ForeachInMetadataInfo, info);
+	}
+
 	g_free (dirname);
 	g_free (basename);
 	g_free (escaped_path);
@@ -857,6 +820,25 @@ tracker_data_update_enable_volume (const gchar *udi,
 	}
 }
 
+void
+tracker_data_update_reset_volume (guint32 volume_id)
+{
+	TrackerDBInterface *iface;
+	gchar *volume_id_str;
+
+	/* NOTE: The default volume id 1 is not to be changed */
+	g_return_if_fail (volume_id > 1);
+
+	iface = tracker_db_manager_get_db_interface (TRACKER_DB_COMMON);
+
+	volume_id_str = tracker_guint32_to_string (volume_id);
+	tracker_db_interface_execute_procedure (iface, NULL,
+						"UpdateVolumeDisabledDate",
+						volume_id_str,
+						NULL);
+	g_free (volume_id_str);
+}
+
 void
 tracker_data_update_disable_volume (const gchar *udi)
 {
