@@ -237,7 +237,12 @@ read_exif (const unsigned char *buffer,
 	ExifData *exif;
 	TagType  *p;
 
-	exif = exif_data_new_from_data ((unsigned char *) buffer, len);
+	exif = exif_data_new();
+	exif_data_set_option (exif, EXIF_DATA_OPTION_IGNORE_UNKNOWN_TAGS);
+	exif_data_unset_option (exif, EXIF_DATA_OPTION_FOLLOW_SPECIFICATION);
+	exif_data_set_option (exif, EXIF_DATA_OPTION_DONT_CHANGE_MAKER_NOTE);
+
+	exif_data_load_data (exif, (unsigned char *) buffer, len);
 
 	for (p = tags; p->name; ++p) {
 		ExifEntry *entry;
@@ -265,6 +270,8 @@ read_exif (const unsigned char *buffer,
 			}
 		}
 	}
+	
+	exif_data_free (exif);
 }
 
 #endif /* HAVE_LIBEXIF */
@@ -274,38 +281,27 @@ static void
 extract_jpeg (const gchar *filename,
 	      GHashTable  *metadata)
 {
-	struct stat  fstatbuf;
-	size_t	     size;
-
 	struct jpeg_decompress_struct  cinfo;
 	struct tej_error_mgr	       tejerr;
 	struct jpeg_marker_struct     *marker;
-	FILE			      *jpeg;
-	gint			       fd_jpeg;
+	FILE			      *f;
+	goffset                        size;
 
-	if ((fd_jpeg = g_open (filename, O_RDONLY)) == -1) {
-		return;
-	}
+	size = tracker_file_get_size (filename);
 
-	if (stat (filename, &fstatbuf) == -1) {
-		close(fd_jpeg);
-		return;
-	}
-
-	/* Check size at least SOI+JFIF without thumb */
-	size = fstatbuf.st_size;
 	if (size < 18) {
-		close (fd_jpeg);
 		return;
 	}
 
-	if ((jpeg = fdopen (fd_jpeg, "rb"))) {
+	f = tracker_file_open (filename, "rb", FALSE);
+
+	if (f) {
 		gchar *str;
 		gsize  len;
 #ifdef HAVE_LIBIPTCDATA
 		gsize  offset;
 		gsize  sublen;
-#endif /* HAVE_LIBEXIF */
+#endif /* HAVE_LIBIPTCDATA */
 
 		cinfo.err = jpeg_std_error (&tejerr.jpeg);
 		tejerr.jpeg.error_exit = tracker_extract_jpeg_error_exit;
@@ -319,7 +315,7 @@ extract_jpeg (const gchar *filename,
 		jpeg_save_markers (&cinfo, JPEG_APP0 + 1, 0xFFFF);
 		jpeg_save_markers (&cinfo, JPEG_APP0 + 13, 0xFFFF);
 		
-		jpeg_stdio_src (&cinfo, jpeg);
+		jpeg_stdio_src (&cinfo, f);
 		
 		jpeg_read_header (&cinfo, TRUE);
 		
@@ -414,9 +410,7 @@ extract_jpeg (const gchar *filename,
 
 		jpeg_destroy_decompress (&cinfo);
 	fail:
-		fclose (jpeg);
-	} else {
-		close (fd_jpeg);
+		tracker_file_close (f, FALSE);
 	}
 }
 
