@@ -82,7 +82,7 @@
 
 #define TRACKER_INDEXER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_INDEXER, TrackerIndexerPrivate))
 
-#define FILES_REMAINING_THRESHOLD   10000
+#define FILES_REMAINING_THRESHOLD   1000
 #define MAX_FLUSH_FREQUENCY         60
 #define MIN_FLUSH_FREQUENCY         1
 
@@ -198,6 +198,7 @@ enum {
 	MODULE_FINISHED,
 	PAUSED,
 	CONTINUED,
+	INDEXING_ERROR,
 	LAST_SIGNAL
 };
 
@@ -213,6 +214,10 @@ static void	state_check	       (TrackerIndexer	    *indexer);
 static void     check_finished         (TrackerIndexer      *indexer,
 					gboolean             interrupted);
 
+static void     item_erase             (TrackerIndexer      *indexer,
+					TrackerService      *service,
+					guint32              service_id);
+
 static gboolean item_process           (TrackerIndexer      *indexer,
 					PathInfo            *info,
 					const gchar         *dirname,
@@ -502,6 +507,10 @@ index_flushing_notify_cb (GObject        *object,
 
 	state = indexer->private->state;
 
+	if ((state & TRACKER_INDEXER_STATE_CLEANUP) != 0) {
+		return;
+	}
+
 	if ((state & TRACKER_INDEXER_STATE_STOPPED) != 0 &&
 	    !tracker_db_index_get_flushing (indexer->private->file_index) &&
 	    !tracker_db_index_get_flushing (indexer->private->email_index)) {
@@ -525,6 +534,15 @@ index_overloaded_notify_cb (GObject        *object,
 	}
 }
 
+static void
+index_error_received_cb (TrackerDBIndex *index,
+			 const GError   *error,
+			 TrackerIndexer *indexer)
+{
+	g_signal_emit (indexer, signals[INDEXING_ERROR], 0,
+		       error->message, TRUE);
+}
+
 static void
 check_mount_removal (GQueue   *queue,
 		     GFile    *mount_root,
@@ -578,7 +596,8 @@ mount_pre_unmount_cb (GVolumeMonitor *volume_monitor,
 	/* Now cancel current element if it's also in the mount */
 	current_info = g_queue_peek_head (indexer->private->file_queue);
 
-	if (g_file_has_prefix (current_info->file, mount_root)) {
+	if (current_info &&
+	    g_file_has_prefix (current_info->file, mount_root)) {
 		tracker_module_file_cancel (current_info->module_file);
 	}
 
@@ -772,6 +791,15 @@ tracker_indexer_class_init (TrackerIndexerClass *class)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_STRING);
+	signals[INDEXING_ERROR] =
+		g_signal_new ("indexing-error",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (TrackerIndexerClass, indexing_error),
+			      NULL, NULL,
+			      tracker_marshal_VOID__STRING_BOOL,
+			      G_TYPE_NONE,
+			      2, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
 	g_object_class_install_property (object_class,
 					 PROP_RUNNING,
@@ -927,6 +955,9 @@ cleanup_task_start (TrackerIndexer *indexer)
 		/* Open indexes */
 		tracker_db_index_open (indexer->private->file_index);
 		tracker_db_index_open (indexer->private->email_index);
+
+		/* Push all items in thumbnail queue to the thumbnailer */
+		tracker_thumbnailer_queue_send ();
 	}
 }
 
@@ -1037,6 +1068,8 @@ tracker_indexer_init (TrackerIndexer *indexer)
 			  G_CALLBACK (index_flushing_notify_cb), indexer);
 	g_signal_connect (priv->file_index, "notify::overloaded",
 			  G_CALLBACK (index_overloaded_notify_cb), indexer);
+	g_signal_connect (priv->file_index, "error-received",
+			  G_CALLBACK (index_error_received_cb), indexer);
 
 	lindex = tracker_db_index_manager_get_index (TRACKER_DB_INDEX_EMAIL);
 	priv->email_index = g_object_ref (lindex);
@@ -1045,6 +1078,8 @@ tracker_indexer_init (TrackerIndexer *indexer)
 			  G_CALLBACK (index_flushing_notify_cb), indexer);
 	g_signal_connect (priv->email_index, "notify::overloaded",
 			  G_CALLBACK (index_overloaded_notify_cb), indexer);
+	g_signal_connect (priv->email_index, "error-received",
+			  G_CALLBACK (index_error_received_cb), indexer);
 
 	/* Set up databases, these pointers are mostly used to
 	 * start/stop transactions, since TrackerDBManager treats
@@ -1549,7 +1584,7 @@ generate_item_thumbnail (TrackerIndexer        *indexer,
 		file = g_file_new_for_path (path);
 		uri = g_file_get_uri (file);
 
-		tracker_thumbnailer_get_file_thumbnail (uri, mime_type);
+		tracker_thumbnailer_queue_file (uri, mime_type);
 
 		g_object_unref (file);
 		g_free (uri);
@@ -1568,6 +1603,7 @@ item_add_or_update (TrackerIndexer        *indexer,
 	guint32 id;
 	gchar *mount_point = NULL;
 	gchar *service_path;
+	gboolean exists, enabled;
 
 	service = get_service_for_file (info->module_file, info->module);
 
@@ -1575,7 +1611,15 @@ item_add_or_update (TrackerIndexer        *indexer,
 		return;
 	}
 
-	if (tracker_data_query_service_exists (service, dirname, basename, &id, NULL)) {
+	exists = tracker_data_query_service_exists (service, dirname, basename, &id, NULL, &enabled);
+
+	if (exists && !enabled) {
+		/* Service must be deleted before insertion */
+		item_erase (indexer, service, id);
+		exists = FALSE;
+	}
+
+	if (exists) {
 		TrackerDataMetadata *old_metadata_emb, *old_metadata_non_emb;
 		gchar *old_text;
 
@@ -1678,6 +1722,7 @@ item_add_or_update (TrackerIndexer        *indexer,
 				     basename, 
 				     NULL);
 
+#ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     service_path, 
 						     &mount_point,
@@ -1689,7 +1734,7 @@ item_add_or_update (TrackerIndexer        *indexer,
 						       tracker_service_get_name (service),
 						       metadata);
 	}
-
+#endif
 	g_free (mount_point);
 	g_free (service_path);
 }
@@ -1758,6 +1803,7 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 	path = g_file_get_path (file);
 	source_path = g_file_get_path (source_file);
 
+#ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     source_path,
 						     &mount_point,
@@ -1781,7 +1827,7 @@ update_moved_item_removable_device (TrackerIndexer *indexer,
 							      service_name);
 		}
 	}
-
+#endif
 	g_free (mount_point);
 	g_free (source_path);
 	g_free (path);
@@ -1916,6 +1962,7 @@ item_move (TrackerIndexer  *indexer,
 	gchar *dest_dirname, *dest_basename;
 	guint32 service_id, dest_service_id;
 	GHashTable *children = NULL;
+	gboolean exists, enabled;
 
 	service = get_service_for_file (info->module_file, info->module);
 
@@ -1929,11 +1976,14 @@ item_move (TrackerIndexer  *indexer,
 	g_debug ("Moving item from '%s' to '%s'", source_path, path);
 
 	/* Get 'source' ID */
-	if (!tracker_data_query_service_exists (service,
-						dirname,
-						basename,
-						&service_id,
-						NULL)) {
+	exists = tracker_data_query_service_exists (service,
+						    dirname,
+						    basename,
+						    &service_id,
+						    NULL,
+						    &enabled);
+
+	if (!exists || !enabled) {
 		gchar *dest_dirname, *dest_basename;
 		gboolean res;
 
@@ -1957,7 +2007,7 @@ item_move (TrackerIndexer  *indexer,
 					       dest_dirname,
 					       dest_basename,
 					       &dest_service_id,
-					       NULL)) {
+					       NULL, NULL)) {
 		g_message ("Destination file '%s' already existed in database, removing", path);
 
 		/* Item has to be deleted from the database immediately */
@@ -2042,6 +2092,7 @@ item_mark_for_removal (TrackerIndexer *indexer,
 	const gchar *service_type;
 	guint service_id, service_type_id;
 	GHashTable *children = NULL;
+	gboolean exists, enabled;
 
 	g_debug ("Removing item: '%s/%s' (no metadata was given by module)", 
 		 dirname, 
@@ -2060,9 +2111,9 @@ item_mark_for_removal (TrackerIndexer *indexer,
 	service_type = tracker_ontology_get_service_by_id (service_type_id);
 	service = tracker_ontology_get_service_by_name (service_type);
 
-	tracker_data_query_service_exists (service, dirname, basename, &service_id, NULL);
+	exists = tracker_data_query_service_exists (service, dirname, basename, &service_id, NULL, &enabled);
 
-	if (service_id < 1) {
+	if (!exists || !enabled) {
 		g_debug ("  File does not exist anyway "
 			 "(dirname:'%s', basename:'%s')",
 			 dirname, basename);
@@ -2101,7 +2152,7 @@ item_mark_for_removal (TrackerIndexer *indexer,
 
 		g_hash_table_destroy (children);
 	}
-
+#ifdef HAVE_HAL
 	if (tracker_hal_path_is_on_removable_device (indexer->private->hal,
 						     path, 
 						     &mount_point,
@@ -2111,7 +2162,7 @@ item_mark_for_removal (TrackerIndexer *indexer,
 						      path,
 						      tracker_service_get_name (service));
 	}
-
+#endif
 	g_free (mount_point);
 	g_free (path);
 }
@@ -2175,6 +2226,7 @@ handle_metadata_add (TrackerIndexer *indexer,
 	gchar          *joined, *dirname = NULL, *basename = NULL;
 	gchar         **old_contents;
 	gint            len;
+	gboolean        exists, enabled;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
@@ -2214,15 +2266,16 @@ handle_metadata_add (TrackerIndexer *indexer,
 
 	tracker_file_get_path_and_name (uri, &dirname, &basename);
 
-	tracker_data_query_service_exists (service,
-				  dirname,
-				  basename,
-				  &service_id,
-				  NULL);
+	exists = tracker_data_query_service_exists (service,
+						    dirname,
+						    basename,
+						    &service_id,
+						    NULL,
+						    &enabled);
 	g_free (dirname);
 	g_free (basename);
 
-	if (service_id < 1) {
+	if (!exists || !enabled) {
 		g_set_error (error,
 			     g_quark_from_string (TRACKER_INDEXER_ERROR),
 			     TRACKER_INDEXER_ERROR_CODE,
@@ -2314,6 +2367,7 @@ handle_metadata_remove (TrackerIndexer *indexer,
 	TrackerField *field;
 	guint service_id, i;
 	gchar *joined = NULL, *dirname = NULL, *basename = NULL;
+	gboolean exists, enabled;
 
 	check_started (indexer);
 
@@ -2348,12 +2402,14 @@ handle_metadata_remove (TrackerIndexer *indexer,
 
 	tracker_file_get_path_and_name (uri, &dirname, &basename);
 
-	tracker_data_query_service_exists (service, dirname, basename, &service_id, NULL);
+	exists = tracker_data_query_service_exists (service,
+						    dirname, basename,
+						    &service_id, NULL, &enabled);
 
 	g_free (dirname);
 	g_free (basename);
 
-	if (service_id < 1) {
+	if (!exists || !enabled) {
 		g_set_error (error,
 			     g_quark_from_string (TRACKER_INDEXER_ERROR),
 			     TRACKER_INDEXER_ERROR_CODE,
@@ -2412,14 +2468,15 @@ handle_metadata_remove (TrackerIndexer *indexer,
 
 static gboolean
 should_change_index_for_file (TrackerIndexer *indexer,
-			      PathInfo        *info,
-			      const gchar	  *dirname,
-			      const gchar	  *basename)
+			      PathInfo       *info,
+			      const gchar    *dirname,
+			      const gchar    *basename)
 {
 	TrackerService *service;
 	gchar *path;
 	guint64 current_mtime;
 	time_t db_mtime;
+	gboolean enabled;
 
 	service = get_service_for_file (info->module_file, info->module);
 
@@ -2434,7 +2491,16 @@ should_change_index_for_file (TrackerIndexer *indexer,
 						dirname,
 						basename,
 						NULL,
-						&db_mtime)) {
+						&db_mtime,
+						&enabled)) {
+		return TRUE;
+	}
+
+	/* Service exists, but is disabled. In this case
+	 * we want to reindex, so the item will be deleted
+	 * afterwards before inserting the new data.
+	 */
+	if (!enabled) {
 		return TRUE;
 	}
 
