@@ -43,22 +43,32 @@
 
 #define TRACKER_DAEMON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_DAEMON, TrackerDaemonPrivate))
 
+#define TRACKER_TYPE_G_STRV_ARRAY  (dbus_g_type_get_collection ("GPtrArray", G_TYPE_STRV))
+
+/* In seconds (5 minutes for now) */
+#define STATS_CACHE_LIFETIME 300
+
 typedef struct {
 	TrackerConfig	 *config;
 	TrackerProcessor *processor;
 	DBusGProxy	 *indexer_proxy;
-	GPtrArray	 *last_stats;
+
+	GHashTable       *stats_cache;
+	guint             stats_cache_timeout_id;
 } TrackerDaemonPrivate;
 
 enum {
 	INDEX_STATE_CHANGE,
 	INDEX_FINISHED,
 	INDEX_PROGRESS,
+	INDEXING_ERROR,
 	SERVICE_STATISTICS_UPDATED,
 	LAST_SIGNAL
 };
 
-static void tracker_daemon_finalize (GObject	 *object);
+static void        tracker_daemon_finalize (GObject       *object);
+static gboolean    stats_cache_timeout     (gpointer       user_data);
+static GHashTable *stats_cache_get_latest  (void);
 
 static guint signals[LAST_SIGNAL] = {0};
 
@@ -114,113 +124,79 @@ tracker_daemon_class_init (TrackerDaemonClass *klass)
 			      G_TYPE_INT,
 			      G_TYPE_INT,
 			      G_TYPE_DOUBLE);
+	signals[INDEXING_ERROR] =
+		g_signal_new ("indexing-error",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0, NULL, NULL,
+			      tracker_marshal_VOID__STRING_BOOLEAN,
+			      G_TYPE_NONE,
+			      2, G_TYPE_STRING, G_TYPE_BOOLEAN);
 	signals[SERVICE_STATISTICS_UPDATED] =
 		g_signal_new ("service-statistics-updated",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      tracker_marshal_VOID__BOXED,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE,
 			      1,
-			      G_TYPE_STRV);
+			      TRACKER_TYPE_G_STRV_ARRAY);
 
 	g_type_class_add_private (object_class, sizeof (TrackerDaemonPrivate));
 }
 
 static void
-clean_last_stats (TrackerDaemonPrivate *priv)
+indexer_started_cb (DBusGProxy *proxy,
+		    gpointer    user_data)
 {
-	if (priv->last_stats) {
-		g_ptr_array_foreach (priv->last_stats, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (priv->last_stats, TRUE);
+	TrackerDaemonPrivate *priv;
+
+	priv = TRACKER_DAEMON_GET_PRIVATE (user_data);
+
+	/* Make sure we have the cache timeout set up */
+	if (priv->stats_cache_timeout_id != 0) {
+		return;
 	}
+
+	g_message ("Starting statistics cache timeout");
+	priv->stats_cache_timeout_id = 
+		g_timeout_add_seconds (STATS_CACHE_LIFETIME,
+				       stats_cache_timeout,
+				       user_data);
 }
 
 static void
 indexer_finished_cb (DBusGProxy *proxy,
 		     gdouble	 seconds_elapsed,
+		     guint	 items_processed,
 		     guint	 items_done,
 		     gboolean	 interrupted,
 		     gpointer	 user_data)
 {
-	GObject		     *daemon;
 	TrackerDaemonPrivate *priv;
-	TrackerDBInterface   *iface;
-	TrackerDBResultSet   *result_set;
-	GPtrArray	     *values = NULL;
-	GPtrArray	     *new_stats;
-	GStrv		      strv;
-	guint		      i;
 
-	daemon = tracker_dbus_get_object (TRACKER_TYPE_DAEMON);
-	priv = TRACKER_DAEMON_GET_PRIVATE (daemon);
-	iface = tracker_db_manager_get_db_interface_by_service (TRACKER_DB_FOR_FILE_SERVICE);
+	tracker_daemon_signal_statistics ();
 
-	/* GetStats has asc in its query. Therefore we don't have to
-	 * lookup the in a to compare in b, just compare index based.
-	 * Maybe we want to change this nonetheless later?
-	 */
-	result_set = tracker_data_manager_exec_proc (iface, "GetStats", 0);
-	new_stats = tracker_dbus_query_result_to_ptr_array (result_set);
+	priv = TRACKER_DAEMON_GET_PRIVATE (user_data);
 
-	if (result_set) {
-		g_object_unref (result_set);
+	if (priv->stats_cache_timeout_id == 0) {
+		return;
 	}
 
-	if (priv->last_stats && new_stats) {
-		for (i = 0; i < priv->last_stats->len && i < new_stats->len; i++) {
-			GStrv str1;
-			GStrv str2;
+	g_message ("Stopping statistics cache timeout");
+	g_source_remove (priv->stats_cache_timeout_id);
+	priv->stats_cache_timeout_id = 0;
+}
 
-			str1 = g_ptr_array_index (priv->last_stats, i);
-			str2 = g_ptr_array_index (new_stats, i);
-
-			if (!str1[0] || !str1[1] || !str2[1]) {
-				continue;
-			}
-
-			if (strcmp (str1[1], str2[1]) != 0) {
-				if (!values) {
-					values = g_ptr_array_new ();
-				}
-
-				g_ptr_array_add (values, g_strdup (str1[0]));
-			}
-		}
-	} else if (new_stats) {
-		for (i = 0; i < new_stats->len; i++) {
-			GStrv str;
-
-			str = g_ptr_array_index (new_stats, i);
-
-			if (!str[0]) {
-				continue;
-			}
-
-			g_ptr_array_add (values, g_strdup (str[0]));
-		}
-	}
-
-	clean_last_stats (priv);
-	priv->last_stats = new_stats;
-
-	if (values && values->len > 0) {
-		strv = g_new0 (gchar*, values->len + 1);
-
-		for (i = 0 ; i < values->len; i++) {
-			strv[i] = g_ptr_array_index (values, i);
-		}
-
-		g_ptr_array_free (values, TRUE);
-		strv[i] = NULL;
-	} else {
-		strv = g_new0 (gchar*, 1);
-		strv[0] = NULL;
-	}
-
-	g_signal_emit (daemon, signals[SERVICE_STATISTICS_UPDATED], 0, strv);
-	g_strfreev (strv);
+static void
+indexing_error_cb (DBusGProxy    *proxy,
+		   const gchar   *reason,
+		   gboolean       requires_reindex,
+		   TrackerDaemon *daemon)
+{
+	g_signal_emit (daemon, signals[INDEXING_ERROR], 0,
+		       reason, requires_reindex);
 }
 
 static void
@@ -228,7 +204,6 @@ tracker_daemon_init (TrackerDaemon *object)
 {
 	TrackerDaemonPrivate *priv;
 	TrackerDBInterface   *iface;
-	TrackerDBResultSet   *result_set;
 	DBusGProxy           *proxy;
 
 	priv = TRACKER_DAEMON_GET_PRIVATE (object);
@@ -236,19 +211,28 @@ tracker_daemon_init (TrackerDaemon *object)
 	proxy = tracker_dbus_indexer_get_proxy ();
 	priv->indexer_proxy = g_object_ref (proxy);
 
+	dbus_g_proxy_connect_signal (proxy, "Started",
+				     G_CALLBACK (indexer_started_cb),
+				     object,
+				     NULL);
 	dbus_g_proxy_connect_signal (proxy, "Finished",
 				     G_CALLBACK (indexer_finished_cb),
 				     object,
 				     NULL);
+	dbus_g_proxy_connect_signal (proxy, "IndexingError",
+				     G_CALLBACK (indexing_error_cb),
+				     object,
+				     NULL);
 
-	iface = tracker_db_manager_get_db_interface_by_service (TRACKER_DB_FOR_FILE_SERVICE);
+	iface = tracker_db_manager_get_db_interface (TRACKER_DB_COMMON);
 
-	result_set = tracker_data_manager_exec_proc (iface, "GetStats", 0);
-	priv->last_stats = tracker_dbus_query_result_to_ptr_array (result_set);
+	/* Do first time stats lookup */
+	priv->stats_cache = stats_cache_get_latest ();
 
-	if (result_set) {
-		g_object_unref (result_set);
-	}
+	priv->stats_cache_timeout_id = 
+		g_timeout_add_seconds (STATS_CACHE_LIFETIME,
+				       stats_cache_timeout,
+				       object);
 }
 
 static void
@@ -260,11 +244,23 @@ tracker_daemon_finalize (GObject *object)
 	daemon = TRACKER_DAEMON (object);
 	priv = TRACKER_DAEMON_GET_PRIVATE (daemon);
 
-	clean_last_stats (priv);
+	if (priv->stats_cache_timeout_id != 0) {
+		g_source_remove (priv->stats_cache_timeout_id);
+	}
 
+	dbus_g_proxy_disconnect_signal (priv->indexer_proxy, "Started",
+					G_CALLBACK (indexer_started_cb),
+					daemon);
 	dbus_g_proxy_disconnect_signal (priv->indexer_proxy, "Finished",
 					G_CALLBACK (indexer_finished_cb),
-					NULL);
+					daemon);
+	dbus_g_proxy_disconnect_signal (priv->indexer_proxy, "IndexingError",
+					G_CALLBACK (indexing_error_cb),
+					daemon);
+
+	if (priv->stats_cache) {
+		g_hash_table_unref (priv->stats_cache);
+	}
 
 	g_object_unref (priv->indexer_proxy);
 
@@ -330,24 +326,22 @@ tracker_daemon_get_status (TrackerDaemon	  *object,
 			   DBusGMethodInvocation  *context,
 			   GError		 **error)
 {
-	TrackerDaemonPrivate *priv;
-	gchar		     *status;
-	guint		      request_id;
+	guint  request_id;
+	gchar *status;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
-	priv = TRACKER_DAEMON_GET_PRIVATE (object);
-
+	tracker_dbus_request_block_hooks ();
 	tracker_dbus_request_new (request_id,
 				  "DBus request to get daemon status");
 
 	status = g_strdup (tracker_status_get_as_string ());
 
 	dbus_g_method_return (context, status);
-
 	g_free (status);
 
 	tracker_dbus_request_success (request_id);
+	tracker_dbus_request_unblock_hooks ();
 }
 
 void
@@ -368,15 +362,7 @@ tracker_daemon_get_services (TrackerDaemon	    *object,
 	tracker_dbus_request_new (request_id,
 				  "DBus request to get daemon services");
 
-	/* Here it doesn't matter which one we ask, as long as it has common.db
-	 * attached. The service ones are cached connections, so we can use
-	 * those instead of asking for an individual-file connection (like what
-	 * the original code had)
-	 */
-
-	/* iface = tracker_db_manager_get_db_interfaceX (TRACKER_DB_COMMON); */
-
-	iface = tracker_db_manager_get_db_interface_by_service (TRACKER_DB_FOR_FILE_SERVICE);
+	iface = tracker_db_manager_get_db_interface (TRACKER_DB_COMMON);
 
 	result_set = tracker_data_manager_exec_proc (iface, "GetServices", 0);
 	values = tracker_dbus_query_result_to_hash_table (result_set);
@@ -392,44 +378,190 @@ tracker_daemon_get_services (TrackerDaemon	    *object,
 	tracker_dbus_request_success (request_id);
 }
 
+static void
+stats_cache_filter_dups_func (gpointer data,
+			      gpointer user_data)
+{
+	GHashTable *values;
+	GStrv       strv;
+	gpointer    p;
+	gint        count;
+
+	/* FIXME: There is a really shit bug here that needs
+	 * fixing. If a file has "Files" as its main category
+	 * and not its *parent* category, then we end up with
+	 * 2 "Files" listings. This sounds like a bug with
+	 * the indexer's insert mechanisms. So far this seems
+	 * to only happen for removable media files
+	 *
+	 * For now, we will concatenate values to sort out the
+	 * duplicates: 
+	 */
+	strv = data;
+	values = user_data;
+
+	count = atoi (strv[1]);
+
+	p = g_hash_table_lookup (values, strv[0]);
+
+	if (G_UNLIKELY (p)) {
+		count += GPOINTER_TO_INT (p);
+	}
+
+	g_hash_table_replace (values, 
+			      g_strdup (strv[0]), 
+			      GINT_TO_POINTER (count));
+}
+
+static GHashTable *
+stats_cache_get_latest (void)
+{
+	TrackerDBResultSet *result_set;
+	TrackerDBInterface *iface;
+	GHashTable         *services;
+	GHashTable         *values;
+	GHashTableIter      iter;
+	gpointer            key, value;
+	guint               i;
+	const gchar        *services_to_fetch[3] = { 
+		TRACKER_DB_FOR_FILE_SERVICE, 
+		TRACKER_DB_FOR_EMAIL_SERVICE, 
+		NULL 
+	};
+
+	/* Set up empty list of services because SQL queries won't give us 0 items. */
+	g_message ("Requesting statistics from database for an accurate signal");
+
+	iface = tracker_db_manager_get_db_interface (TRACKER_DB_COMMON);
+	result_set = tracker_data_manager_exec_proc (iface, "GetServices", 0);
+	services = tracker_dbus_query_result_to_hash_table (result_set);
+
+	if (result_set) {
+		g_object_unref (result_set);
+	}
+
+	values = g_hash_table_new_full (g_str_hash,
+					g_str_equal,
+					g_free,
+					NULL);
+
+	/* Put services with 0 counts into new hash table */
+	g_hash_table_iter_init (&iter, services);
+
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		g_hash_table_replace (values, g_strdup (key), GINT_TO_POINTER (0));
+	}
+
+	g_hash_table_unref (services);
+
+	/* Populate with real stats */
+	for (i = 0; services_to_fetch[i]; i++) {		
+		TrackerDBInterface *iface;
+		GPtrArray          *stats, *parent_stats;
+
+		iface = tracker_db_manager_get_db_interface_by_service (services_to_fetch[i]);
+
+		/* GetStats has asc in its query. Therefore we don't have to
+		 * lookup the in a to compare in b, just compare index based.
+		 * Maybe we want to change this nonetheless later?
+		 */
+		result_set = tracker_data_manager_exec_proc (iface, "GetStats", 0);
+		stats = tracker_dbus_query_result_to_ptr_array (result_set);
+
+		if (result_set) {
+			g_object_unref (result_set);
+		}
+
+		result_set = tracker_data_manager_exec_proc (iface, "GetStatsForParents", 0);
+		parent_stats = tracker_dbus_query_result_to_ptr_array (result_set);
+
+		if (result_set) {
+			g_object_unref (result_set);
+		}
+		
+		g_ptr_array_foreach (stats, stats_cache_filter_dups_func, values);
+		g_ptr_array_foreach (parent_stats, stats_cache_filter_dups_func, values);
+
+		g_ptr_array_free (parent_stats, TRUE);
+		g_ptr_array_free (stats, TRUE);
+	}
+
+	return values;
+}
+
+static gboolean 
+stats_cache_timeout (gpointer user_data)
+{
+	g_message ("Statistics cache has expired, updating...");
+
+	tracker_dbus_indexer_check_is_paused ();
+	tracker_daemon_signal_statistics ();
+
+	return TRUE;
+}
+
+static gint
+stats_cache_sort_func (gconstpointer a,
+		       gconstpointer b)
+{
+	
+	const GStrv *strv_a = (GStrv *) a;
+	const GStrv *strv_b = (GStrv *) b;
+
+	g_return_val_if_fail (strv_a != NULL, 0);
+	g_return_val_if_fail (strv_b != NULL, 0);
+
+	return g_strcmp0 (*strv_a[0], *strv_b[0]);
+}
+
 void
 tracker_daemon_get_stats (TrackerDaemon		 *object,
 			  DBusGMethodInvocation  *context,
 			  GError		**error)
 {
-	TrackerDBInterface *iface;
-	TrackerDBResultSet *result_set;
-	guint		    request_id;
-	GPtrArray	   *values = NULL;
+	TrackerDaemonPrivate *priv;
+	guint		      request_id;
+	GPtrArray            *values;
+	GHashTableIter        iter;
+	gpointer              key, value;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
+	tracker_dbus_request_block_hooks ();
 	tracker_dbus_request_new (request_id,
 				  "DBus request to get daemon service stats");
 
-	/* Here it doesn't matter which one we ask, as long as it has common.db
-	 * attached. The service ones are cached connections, so we can use
-	 * those instead of asking for an individual-file connection (like what
-	 * the original code had)
-	 */
+	priv = TRACKER_DAEMON_GET_PRIVATE (object);
 
-	/* iface = tracker_db_manager_get_db_interfaceX (TRACKER_DB_COMMON); */
+	values = g_ptr_array_new ();
 
-	iface = tracker_db_manager_get_db_interface_by_service (TRACKER_DB_FOR_FILE_SERVICE);
+	g_hash_table_iter_init (&iter, priv->stats_cache);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		GStrv        strv;
+		const gchar *service_type;
+		gint         count;
 
-	result_set = tracker_data_manager_exec_proc (iface, "GetStats", 0);
-	values = tracker_dbus_query_result_to_ptr_array (result_set);
+		service_type = key;
+		count = GPOINTER_TO_INT (value);
 
-	if (result_set) {
-		g_object_unref (result_set);
+		strv = g_new (gchar*, 3);
+		strv[0] = g_strdup (service_type);
+		strv[1] = g_strdup_printf ("%d", count);
+		strv[2] = NULL;
+
+		g_ptr_array_add (values, strv);
 	}
 
+	/* Sort result so it is alphabetical */
+	g_ptr_array_sort (values, stats_cache_sort_func);
+
 	dbus_g_method_return (context, values);
 
 	g_ptr_array_foreach (values, (GFunc) g_strfreev, NULL);
 	g_ptr_array_free (values, TRUE);
 
 	tracker_dbus_request_success (request_id);
+	tracker_dbus_request_unblock_hooks ();
 }
 
 void
@@ -649,3 +781,87 @@ tracker_daemon_prompt_index_signals (TrackerDaemon	    *object,
 
 	tracker_dbus_request_success (request_id);
 }
+
+void
+tracker_daemon_signal_statistics (void)
+{
+	GObject		     *daemon;
+	TrackerDaemonPrivate *priv;
+	GHashTable           *stats;
+	GHashTableIter        iter;
+	gpointer              key, value;
+	GPtrArray            *values;
+
+	daemon = tracker_dbus_get_object (TRACKER_TYPE_DAEMON);
+	priv = TRACKER_DAEMON_GET_PRIVATE (daemon);
+
+	/* Get latest */
+	stats = stats_cache_get_latest ();
+
+	/* There are 3 situations here:
+	 *  - 1. No new stats
+	 *       Action: Do nothing
+	 *  - 2. New stats and old stats
+	 *       Action: Check what has changed and emit new stats
+	 */
+
+	g_message ("Checking for statistics changes and signalling clients...");
+
+	/* Situation #1 */
+	if (g_hash_table_size (stats) < 1) {
+		g_hash_table_unref (stats);
+		g_message ("  No new statistics, doing nothing");
+		return;
+	}
+
+	/* Situation #2 */
+	values = g_ptr_array_new ();
+
+	g_hash_table_iter_init (&iter, stats);
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		const gchar  *service_type;
+		gpointer      data;
+		gint          old_count, new_count;
+		
+		service_type = key;
+		new_count = GPOINTER_TO_INT (value);
+			
+		data = g_hash_table_lookup (priv->stats_cache, service_type);
+		old_count = GPOINTER_TO_INT (data);
+		
+		if (old_count != new_count) {
+			GStrv strv;
+
+			g_message ("  Updating '%s' with new count:%d, old count:%d, diff:%d", 
+				   service_type,
+				   new_count,
+				   old_count,
+				   new_count - old_count);
+			
+			g_hash_table_replace (priv->stats_cache, 
+					      g_strdup (service_type), 
+					      GINT_TO_POINTER (new_count));
+
+			strv = g_new (gchar*, 3);
+			strv[0] = g_strdup (service_type);
+			strv[1] = g_strdup_printf ("%d", new_count);
+			strv[2] = NULL;
+			
+			g_ptr_array_add (values, strv);
+		}
+	}
+
+	g_hash_table_unref (stats);
+
+	if (values->len > 0) {
+		/* Make sure we sort the results first */
+		g_ptr_array_sort (values, stats_cache_sort_func);
+		
+		g_signal_emit (daemon, signals[SERVICE_STATISTICS_UPDATED], 0, values);
+	} else {
+		g_message ("  No changes in the statistics");
+	}
+	
+	g_ptr_array_foreach (values, (GFunc) g_strfreev, NULL);
+	g_ptr_array_free (values, TRUE);
+}
